#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct 19 17:11:30 2020

@author: 
"""

import numpy as np
import tensorflow as tf
import tensorflow.keras as keras

import metrics

    
class Classifier(tf.keras.Model):
    
    def count_trainable_params(self):
        
        sizes = [np.prod(var.shape) for var in self.trainable_variables]
        total_size = np.asscalar(np.sum(sizes))
        return total_size

    def set_learning_rate(self, learning_rate):

        tf.keras.backend.set_value(self.optimizer.learning_rate, learning_rate)
        
        #tf.keras.backend.set_value(self.structure_optimizer.learning_rate, learning_rate * 0.01)
        
    def _build_train_step(self):
        
        #tf.nn.compute_average_loss
        self._crossentropy = tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True, reduction=tf.keras.losses.Reduction.NONE)
        self.optimizer = tf.keras.optimizers.SGD(learning_rate=0.1, momentum=0.9, nesterov=True)
        
        #self.structure_optimizer = tf.keras.optimizers.Adam(learning_rate=0.1*0.0025)


    def _build_metrics(self):
        
        self.train_tracker = []
        crossentropy_tracker = keras.metrics.SparseCategoricalCrossentropy(from_logits=True,name='Crossentropy')
        #accuracy_tracker = keras.metrics.SparseCategoricalAccuracy()
        error_tracker = metrics.SparseCategoricalError(name='Error')
        #l2_tracker = keras.metrics.Mean(name='l2')

        self._crossentropy_tracker = crossentropy_tracker
        self._error_tracker = error_tracker
        #self._l2_tracker = l2_tracker

        
        #self._metrics = [crossentropy_tracker, error_tracker, l2_tracker]
        self._metrics = [crossentropy_tracker, error_tracker]

    def compile(self):
        super(Classifier, self).compile()
        self._build_train_step()
        self._build_metrics()
        

        
    def train_step(self, data):
        inputs, targets = data
        
        with tf.GradientTape() as tape:
            # Compute a training loss
            logits = self(inputs, training=True)
            crossentropy = self._crossentropy(y_true=targets, y_pred=logits) 
            crossentropy = tf.nn.compute_average_loss(crossentropy)
            l2_reg = sum(self.losses)
            total_loss  = crossentropy + l2_reg
            
            # Update weights
            grads = tape.gradient(total_loss, self.trainable_variables)
            
            self.optimizer.apply_gradients(zip(grads, self.trainable_variables))        
            
            # model_grads = []; str_grads = []
            # model_vars = []; str_vars = []
            
            # for grad, var in zip(grads, self.trainable_variables):
            #     if var.name is not None and ('ssr_alpha' in var.name or 'ssr_beta' in var.name):
            #         str_grads.append(grad); str_vars.append(var)
            #     else:
            #         model_grads.append(grad); model_vars.append(var)
                    
            # self.optimizer.apply_gradients(zip(model_grads, model_vars))
            # self.structure_optimizer.apply_gradients(zip(str_grads, str_vars))

        self._crossentropy_tracker.update_state(y_true=targets, y_pred=logits)
        self._error_tracker.update_state(y_true=targets, y_pred=logits)
        #self._l2_tracker.update_state(l2_reg)
        
        return {self._crossentropy_tracker.name:self._crossentropy_tracker.result(),
                self._error_tracker.name:self._error_tracker.result()}
                #self._l2_tracker.name:self._l2_tracker.result()}



    def test_step(self, data):
        inputs, targets = data
        
        logits = self(inputs, training=False)
        self._crossentropy_tracker.update_state(y_true=targets, y_pred=logits)
        self._error_tracker.update_state(y_true=targets, y_pred=logits)

        return {self._crossentropy_tracker.name:self._crossentropy_tracker.result(),
                self._error_tracker.name:self._error_tracker.result()}







