#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct 19 17:11:30 2020

@author: 
"""

import tensorflow as tf
import tensorflow.keras as keras
from module import Conv2D, BatchNormalization, Dense
  

        
class CifarInput(keras.layers.Layer):
    
        def __init__(self, filters, kernel_size=(3,3), strides=(1,1)):

            super(CifarInput, self).__init__()
            self.conv = Conv2D(filters=filters, kernel_size=kernel_size, strides=strides, padding='same')

        def call(self, input_tensor):
            
            y = self.conv(input_tensor)
            
            return y
        
        def modules(self):
            
            modules = [self.conv]
            return modules


class Output(keras.layers.Layer):
    
    def __init__(self, units):
        super(Output, self).__init__()
        self.batch_norm = BatchNormalization() 
        self.activation = tf.nn.relu
        self.pooling2d = keras.layers.GlobalAveragePooling2D()
        self.dense = Dense(units)
        
    def call(self, input_tensor, training=False):
        
        y = self.batch_norm(input_tensor, training)
        y = self.activation(y)
        y = self.pooling2d(y)
        y = self.dense(y)
        
        return y
    
    def modules(self):

        modules = [self.batch_norm, self.activation, self.pooling2d, self.dense]
        return modules


class ImageNetInput(keras.layers.Layer):
    
        def __init__(self, filters, kernel_size=(7,7), pool_size=(3,3), strides=(2,2), **kwargs):

            super(ImageNetInput, self).__init__(**kwargs)
            
            self.conv = Conv2D(filters=filters, kernel_size=kernel_size, strides=strides, padding='same', **kwargs)
            self.batch_norm = BatchNormalization()
            self.activation = tf.nn.relu
            self.pooling2d = keras.layers.MaxPooling2D(pool_size=pool_size, strides=strides, padding='same')


        def call(self, input_tensor, training=False):
            
            y = self.conv(input_tensor)
            y = self.batch_norm(y, training=training)
            y = self.activation(y)
            y = self.pooling2d(y)
            
            return y

        def modules(self):
            
            modules = [self.conv, self.batch_norm, self.activation, self.pooling2d]
            
            return modules