#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Oct 22 16:57:19 2020

@author: 
"""

import tensorflow as tf
import tensorflow.keras as keras
import numpy as np 

class DenseNetLearningRateScheduler(keras.callbacks.Callback):
    """Learning rate scheduler which sets the learning rate according to schedule.

  Arguments:
      schedule: a function that takes an epoch index
          (integer, indexed from 0) and current learning rate
          as inputs and returns a new learning rate as output (float).
  """

    def __init__(self, initial_learning_rate=0.1, epochs=300, decay_epoch=[0.5, 0.75]):
        super(DenseNetLearningRateScheduler, self).__init__()
        self.initial_learning_rate = initial_learning_rate
        self.epochs = epochs
        self.decay_epoch = decay_epoch

    def on_epoch_begin(self, epoch, logs=None):


        scheduled_learning_rate = self.initial_learning_rate
        for decay_epoch in self.decay_epoch:
            if epoch >= self.epochs * decay_epoch: 
                scheduled_learning_rate = scheduled_learning_rate / 10
        
        
        self.model.set_learning_rate(scheduled_learning_rate)     
        print("\nEpoch %05d: Learning rate is %6.4f." % (epoch, scheduled_learning_rate))
        
        


class ResNetLearningRateScheduler(keras.callbacks.Callback):
    """Learning rate scheduler which sets the learning rate according to schedule.

  Arguments:
      schedule: a function that takes an epoch index
          (integer, indexed from 0) and current learning rate
          as inputs and returns a new learning rate as output (float).
  """

    def __init__(self, initial_learning_rate=0.1, epochs=200, decay_epoch=[0.4, 0.6]):
        super(ResNetLearningRateScheduler, self).__init__()
        self.initial_learning_rate = initial_learning_rate
        self.epochs = epochs
        self.decay_epoch = decay_epoch

    def on_epoch_begin(self, epoch, logs=None):


        scheduled_learning_rate = self.initial_learning_rate
        for decay_epoch in self.decay_epoch:
            if epoch >= self.epochs * decay_epoch: 
                scheduled_learning_rate = scheduled_learning_rate / 10
        
        
        self.model.set_learning_rate(scheduled_learning_rate)     
        print("\nEpoch %05d: Learning rate is %6.4f." % (epoch, scheduled_learning_rate))
        

class CosLearningRateScheduler(keras.callbacks.Callback):
    """Learning rate scheduler which sets the learning rate according to schedule.

  Arguments:
      schedule: a function that takes an epoch index
          (integer, indexed from 0) and current learning rate
          as inputs and returns a new learning rate as output (float).
  """

    def __init__(self, initial_learning_rate=0.1, steps=300):
        super(CosLearningRateScheduler, self).__init__()
        self.initial_learning_rate = initial_learning_rate
        self.steps = steps
        self.scheduled_learning_rate = initial_learning_rate
        self.count_step = 0
        
        
    def on_epoch_begin(self, epoch, logs=None):
        print("\nEpoch %05d: Learning rate is %6.10f." % (epoch, self.scheduled_learning_rate))


    def on_batch_begin(self, batch, logs=None):

        self.scheduled_learning_rate = 0.5 * self.initial_learning_rate * (1+ np.cos(self.count_step / self.steps * np.pi))
        self.model.set_learning_rate(self.scheduled_learning_rate)     
        self.count_step += 1
        
       