#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct 19 17:11:30 2020

@author: 
"""

import numpy as np
import tensorflow as tf
import tensorflow.keras as keras

from refactoring import refactor

import util

import tf_cifar_dataset
import sparse_densenet as densenet
import learning_rate


util.configure_gpu()


seed = 1
np.random.seed(seed)
tf.random.set_seed(seed)


dataset = 'cifar10'
#dataset = 'cifar100'

if dataset == 'cifar10':
    classes = 10
    input_shape=(32,32,3)
    train_dataset, test_dataset  = tf_cifar_dataset.load_data(dataset, batch_size=64)
    
elif dataset == 'cifar100':
    classes = 100
    input_shape=(32,32,3)
    train_dataset, test_dataset  = tf_cifar_dataset.load_data(dataset, batch_size=64)

epochs = 320
depth = 100

#Cifar10 
sparsity_scale = 0.0000310 # 60% sparsity rate
#sparsity_scale = 0.0000551 # 70% sparsity rate
#sparsity_scale = 0.0000760 # 75% sparsity rate
#sparsity_scale = 0.0001040 # 80% sparsity rate

#Cifar100
#sparsity_scale = 0.0000830  # 60% sparsity rate
#sparsity_scale = 0.0001300  # 70% sparsity rate
#sparsity_scale = 0.0001635  # 75% sparsity rate
#sparsity_scale = 0.0002150  # 80% sparsity rate



source_cfg = {'batch_norm':'differentiable', 'sparsity_scale':sparsity_scale}
denseNet = densenet.build_DenseNet_BC_K12(classes=classes, depth=depth, cfg=source_cfg)

 
inputs = keras.Input(shape=input_shape)
outputs = denseNet(inputs)
denseNet.compile()
denseNet.summary()

callbacks = [learning_rate.DenseNetLearningRateScheduler(epochs=epochs), util.Plot(), util.Plot_Sparsity()]
callbacks.append(util.Plot_Dense_Conn())
callbacks.append(util.Plot_Group())
denseNet.fit(train_dataset, epochs=epochs, validation_data=test_dataset, validation_freq=1, callbacks=callbacks)
   
#genrate a slim model from a trained model and calculate flops.
slim = densenet.build_DenseNet_BC_K12(classes=classes, depth=depth, cfg=None)
flops = refactor(inputs, denseNet, slim)
denseNet.summary()
slim.summary()

print(seed, dataset, depth, sparsity_scale)    

denseNet.evaluate(test_dataset)    
slim.evaluate(test_dataset)    
print('FLOPS: ', flops)
