#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct 19 17:11:30 2020

@author: 
"""

import numpy as np
import tensorflow as tf
import tensorflow.keras as keras

import util
from mobilenet_profile import Profile
import tf_cifar_dataset

from dnw_differentiable import MobileNet
import learning_rate

util.configure_gpu()


seed = 1
np.random.seed(seed)
tf.random.set_seed(seed)


dataset = 'cifar10'
#dataset = 'cifar100'


if dataset == 'cifar10':
    classes = 10
    batch_size = 128
    input_shape = (32,32,3)
    augmentation = True
    train_dataset, test_dataset  = tf_cifar_dataset.load_data(dataset, batch_size=batch_size)
    epoch_length = int(np.ceil(50000 / batch_size))
    
elif dataset == 'cifar100':
    classes = 100
    batch_size = 128
    input_shape=(32,32,3)
    train_dataset, test_dataset  = tf_cifar_dataset.load_data(dataset, batch_size=batch_size)
    epoch_length = int(np.ceil(50000 / batch_size))
    


alpha = 1.0
epochs = 160


#Cifar10
#sparsity_scale = 0.00006000 # 
#sparsity_scale = 0.00006500 # 
sparsity_scale = 0.00007000 # 
#sparsity_scale = 0.00007250 # 
#sparsity_scale = 0.00007500 # 
#sparsity_scale = 0.00008000 # 


dnw_mobilenet = MobileNet(classes=classes, alpha=alpha, sparsity_scale=sparsity_scale, dropout_rate=0.0)
inputs = keras.Input(shape=input_shape)
outputs = dnw_mobilenet(inputs)
dnw_mobilenet.compile()
dnw_mobilenet.summary()

callbacks = [learning_rate.CosLearningRateScheduler(initial_learning_rate=0.1, steps=epoch_length * epochs), util.Plot()]
callbacks.append(Profile())
dnw_mobilenet.fit(train_dataset, epochs=epochs, validation_data=test_dataset, validation_freq=1, callbacks=callbacks)
   

print(seed, dataset, epochs, batch_size, alpha, sparsity_scale)    

flops, params = dnw_mobilenet.flops()

print(flops, params)
print(f"Mult-Adds: {flops / 10 ** 6:.03} M", f"Params: {params / 10 ** 6:.03} M")
