#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct 19 17:11:30 2020

@author: 
"""

import math
import tensorflow as tf
import tensorflow.keras as keras


#https://github.com/liuzhuang13/DenseNetCaffe/issues/2#issuecomment-248655099

#local n = v.kW*v.kH*v.nOutputPlane
#v.weight:normal(0,math.sqrt(2/n))
#https://github.com/liuzhuang13/DenseNetCaffe/issues/10

class Conv2D(keras.layers.Conv2D):
    
    def __init__(self, filters, kernel_size, strides, padding='same', l2=0.00005, use_bias=False, **kwargs):

        #https://github.com/facebookarchive/fb.resnet.torch/issues/106
        #kernel_initializer = keras.initializers.he_normal()
        n = filters * kernel_size[0] * kernel_size[1]
        stddev = math.sqrt(2/n)
        kernel_initializer = keras.initializers.RandomNormal(mean=0.0, stddev=stddev)
        
        kernel_regularizer = keras.regularizers.l2(l=l2)

        super(Conv2D, self).__init__(filters=filters, 
                                     kernel_size=kernel_size, 
                                     strides=strides, 
                                     padding=padding,
                                     kernel_initializer=kernel_initializer,
                                     kernel_regularizer=kernel_regularizer,
                                     use_bias=use_bias, 
                                     **kwargs)
    


    # def kernel_group_norm(self):
    #     kernel = self.kernel
    #     square = tf.square(kernel)
    #     sum_square = tf.reduce_sum(square, axis=[0, 1, 3], keepdims=True)
    #     norm = tf.sqrt(sum_square)

    #     return norm
    
        
    
    # def proximal_update(self, scale):

    #     kernel = self.kernel
    #     norm = self.kernel_group_norm()
    #     safe_norm = tf.where(tf.greater(norm, 0), norm, tf.ones_like(norm))
    #     updated = tf.nn.relu(1- scale/safe_norm) * kernel
        
    #     mask = tf.where(tf.greater(norm, 0), tf.ones_like(norm), tf.zeros_like(norm))    
        
    #     self.kernel.assign(updated)
        
    #     return mask
        
    
class DepthwiseConv2D(keras.layers.DepthwiseConv2D):
    def __init__(self, filters, kernel_size, strides, depth_multiplier=1, padding='same', l2=0.0005, use_bias=False, **kwargs):
        
        n = filters * kernel_size[0] * kernel_size[1]
        stddev = math.sqrt(2/n)
        initializer = keras.initializers.RandomNormal(mean=0.0, stddev=stddev)
        regularizer = keras.regularizers.l2(l=l2)

        super(DepthwiseConv2D, self).__init__(kernel_size=kernel_size, 
                                              strides=strides,
                                              depth_multiplier=depth_multiplier,
                                              padding=padding,
                                              depthwise_initializer=initializer, 
                                              depthwise_regularizer=regularizer,
                                              use_bias=use_bias,
                                              **kwargs)

#https://github.com/liuzhuang13/DenseNet/blob/master/models/densenet.lua
   # for k,v in pairs(model:findModules('nn.Linear')) do
   #    v.bias:zero()
   # end

class Dense(keras.layers.Dense):
    
    def __init__(self, units, l2=0.00005, use_bias=True, **kwargs):

        
        kernel_initializer = tf.keras.initializers.VarianceScaling(scale=1.0/3.0, mode='fan_in', distribution='uniform')
        bias_initializer = tf.keras.initializers.zeros()
        regularizer = keras.regularizers.l2(l=l2)

        super(Dense, self).__init__(units=units, 
                            kernel_initializer=kernel_initializer,
                            kernel_regularizer=regularizer,
                            use_bias=use_bias,
                            bias_initializer=bias_initializer,
                            bias_regularizer=regularizer,                         
                            **kwargs)
        
       
        
class BatchNormalization(keras.layers.BatchNormalization):
  
    def __init__(self, momentum=0.9, epsilon=1.0e-5, l2=0.00005, center=True, scale=True, **kwargs):
        
        #https://github.com/liuzhuang13/DenseNetCaffe/issues/2#issuecomment-247745029
        regularizer = keras.regularizers.l2(l=l2) # reg for center(beta) and scale(gamma)
        
        super(BatchNormalization, self).__init__(
            momentum=momentum,
            epsilon=epsilon, 
            center=center,
            scale=scale,
            beta_regularizer=regularizer, # center
            gamma_regularizer=regularizer, # scale
            **kwargs)
        
        

            
