#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct 19 17:11:30 2020

@author: 
"""

import numpy as np
import tensorflow as tf
import tensorflow.keras as keras

from sparse_batch_norm import SparseBatchNormalization, Selection
from module import Conv2D, Dense, BatchNormalization

from flops_calculation import get_flops


def get_layers(model):

    modules = model.modules()
    layers = []
    
    while len(modules) > 0:
        module = modules.pop(0)
        if hasattr(module, 'modules'):
            modules = module.modules() + modules
        else:
            layers.append(module)
    
    return layers


    
def copy_batch_norm(old, new, indices=None):
    
    if hasattr(old, 'batch_norm_params'):
        moving_mean, moving_variance, gamma, beta = old.batch_norm_params()
    else: 
        moving_mean = old.moving_mean
        moving_variance = old.moving_variance
        gamma = old.gamma
        beta = old.beta

    moving_mean = moving_mean if indices is None else tf.gather(moving_mean, indices) 
    moving_variance = moving_variance if indices is None else tf.gather(moving_variance, indices)
    gamma =  gamma if indices is None else tf.gather(gamma, indices)
    beta = beta if indices is None else tf.gather(beta, indices) 
    
    new.moving_mean.assign(moving_mean)
    new.moving_variance.assign(moving_variance)
    new.gamma.assign(gamma)
    new.beta.assign(beta)

    
def copy_conv2d(old, new, incoming=None, outcoming=None):
    
    kernel = old.kernel
    kernel = kernel if incoming is None else tf.gather(kernel, incoming, axis=2) # axis=2, incoming channel axis
    kernel = kernel if outcoming is None else tf.gather(kernel, outcoming, axis=3) # axis=3, outcoming channel axis 
    
    new.kernel.assign(kernel)

def copy_dense(old, new, indices=None):
    kernel = tf.gather(old.kernel, indices, axis=0)
    bias = old.bias
    new.kernel.assign(kernel)
    new.bias.assign(bias)


def copy_shortcut_transform(old, new):
    pass
    
    
def channel_selection(batch_norm):
    
    indices = batch_norm.select_channel()
    return indices

    # alpha, _, _ = batch_norm.get_alpha_beta()
    # alpha = np.squeeze(alpha.numpy())
    # indices = np.squeeze(np.where(alpha>0))
    # return indices


def reconstrut(original, slim):
    
    original_layers = get_layers(original)
    slim_layers = get_layers(slim)

    for i, (original, slim) in enumerate(zip(original_layers, slim_layers)):
        
        if isinstance(original, SparseBatchNormalization):
    
            indices = channel_selection(original)
            
            original_prev = original_layers[i-1]
            slim_prev = slim_layers[i-1]
            
            if isinstance(original_prev, Selection):
                #slim_prev.indices = indices
                slim_prev.set_index(indices)
                
            elif isinstance(original_prev, Conv2D):
                #slim_prev.filters = indices.size
                slim_prev.filters = tf.size(indices)


def copy_params(original, slim):

    original_layers = get_layers(original)
    slim_layers = get_layers(slim)

    for i, (old, new) in enumerate(zip(original_layers, slim_layers)):
    
        if isinstance(old, Conv2D):
    
            incoming = None; outcoming = None
            if i > 0 and isinstance(original_layers[i-1], SparseBatchNormalization):
                incoming = channel_selection(original_layers[i-1])
            
            if isinstance(original_layers[i+1], SparseBatchNormalization):
                outcoming = channel_selection(original_layers[i+1])
            
            copy_conv2d(old, new, incoming, outcoming)
    
        if isinstance(old, SparseBatchNormalization):
            indices = channel_selection(old)
            copy_batch_norm(old, new, indices)

        if isinstance(old, BatchNormalization):
            copy_batch_norm(old, new)

    
        if isinstance(old, Dense) and isinstance(original_layers[i-1], SparseBatchNormalization):
            indices = channel_selection(original_layers[i-1])
            copy_dense(old, new, indices)



def refactor(inputs, original, slim):
    reconstrut(original, slim)
    outputs = slim(inputs)
    slim.compile()
    copy_params(original, slim)
    
    slim_model = keras.Model(inputs, outputs)
    flops = get_flops(slim_model, batch_size=1)

    return flops
    
#new_denseNet.evaluate(test_dataset)    







